@@ -1,7 +1,7 @@
 use bevy::prelude::*;
 
 use bevy_ascii_terminal::{render::TerminalRendererFont, TerminalBundle, TerminalPlugin};
-use bevy_pixel_camera::{PixelBorderPlugin, PixelCameraBundle, PixelCameraPlugin};
+use bevy_pixel_camera::{PixelCameraBundle, PixelCameraPlugin};
 
 const FONTS: [&str; 2] = ["alloy_curses_12x12.png", "zx_evolution_8x8.png"];
 #[derive(Default)]
@@ -29,7 +29,7 @@ fn change_font(
     if keys.just_pressed(KeyCode::Space) {
         for mut font in q.iter_mut() {
             font_index.0 = 1 - font_index.0;
-            font.0 = String::from(FONTS[font_index.0]);
+            font.font_name = String::from(FONTS[font_index.0]);
         }
     }
 }
@@ -39,10 +39,8 @@ fn main() {
         .init_resource::<FontIndex>()
         .add_plugins(DefaultPlugins)
         .add_plugin(TerminalPlugin)
+        .insert_resource(ClearColor(Color::BLACK))
         .add_plugin(PixelCameraPlugin)
-        .add_plugin(PixelBorderPlugin {
-            color: Color::BLACK,
-        })
         .add_startup_system(spawn_terminal.system())
         .add_system(change_font.system())
         .run()
